//
// QGIS Includes
//
#include <qgsapplication.h>
#include <qgsproviderregistry.h>
#include <qgssinglesymbolrenderer.h>
#include <qgsmaplayerregistry.h>
#include <qgsvectorlayer.h>
#include <qgsmapcanvas.h>
#include <qgsgeometry.h>
#include <qgsvectordataprovider.h>
#include <qgsmaprenderer.h>
#include <qgsrendercontext.h>
#include <qgslabelattributes.h>
#include <qgsfield.h>
#include <qgsvectorfilewriter.h>
#include <qgscontinuouscolorrenderer.h>
#include <qgsgraduatedsymbolrenderer.h>
#include <qgsuniquevaluerenderer.h>
#include <qgsdistancearea.h>
#include <qgslabel.h>
#include <qgis.h>


//
// Qt Includes
//
#include <QtGui>

int main(int argc, char ** argv) 
{
  // Start the Application
  QgsApplication app(argc, argv, true);

  QString myPluginsDir        = "./plugins";
  QgsProviderRegistry::instance(myPluginsDir);

  QString fname = QFileDialog::getOpenFileName( 0, ("Open map"), ".", ("QGIS project files (*.qgs)") );
  QFile proj_file( fname );
  if (!proj_file.open(QFile::ReadOnly | QFile::Text))
  {
	  qDebug( "unable to open file." );
	  return 1;
  }
  QgsMapCanvas * mMapCanvas = new QgsMapCanvas(0, 0);
  mMapCanvas->enableAntiAliasing(true);
  mMapCanvas->setCanvasColor(QColor(255, 255, 255));
  mMapCanvas->freeze(false);
  QList <QgsMapCanvasLayer> myLayerSet;

  myLayerSet.clear();

  QDomDocument* doc = new QDomDocument("qgis");
  doc->setContent( &proj_file );

  mMapCanvas->readProject( *doc );

  QDomNodeList legendLayers = doc->elementsByTagName("legendlayer");
  QDomNodeList nl = doc->elementsByTagName( "maplayer" );
  for (int i = 0; i < legendLayers.count(); i++)
  {
	  QDomNode nod = legendLayers.item( i ).firstChild().firstChild();
	  QDomElement elem = nod.toElement();
	  QString lid = elem.attribute("layerid");

	  for (int j = 0; j < nl.count(); j++)
	  {
		  if (lid == nl.item( j ).firstChild().firstChild().nodeValue())
		  {
			  QDomNode maplay = nl.item(j);
			  QString type = nl.item( j ).toElement().attribute("type");

			  if (type != "vector")
			  {
				  qDebug("raster layers not supported. Terminating!");
				  return 1;
			  }

			  QgsVectorLayer* newLayer = new QgsVectorLayer();
			  QgsVectorLayer* vl = 0;
			  if ( newLayer->readXML( maplay ) )
			  {
				  qDebug() << "newLayer->isValid" << newLayer->isValid() << "type: " << newLayer->type() << "wkbType: " << newLayer->wkbType();
				  qDebug() << "newLayer renderer: " << newLayer->renderer()->name() << newLayer->renderer();
				  QgsVectorDataProvider * spr = newLayer->dataProvider();
				  QString geomType;
				  QString layerName = newLayer->name();
				  if ( newLayer->geometryType() == QGis::Point )
					  geomType = "Point";
				  if ( newLayer->geometryType() == QGis::Line )
					  geomType = "LineString";
				  if ( newLayer->geometryType() == QGis::Polygon )
					  geomType = "Polygon";
				  if ( newLayer->geometryType() == QGis::UnknownGeometry )
					  qDebug() << "Unknown geometry";

				  vl = new QgsVectorLayer( geomType, layerName, "memory" );
				  qDebug() << "vl is valid: " << vl->isValid() << "type:" << vl->type() << "wkbType: " << vl->wkbType();
				  QgsVectorDataProvider * pr = vl->dataProvider();
				  QgsFeatureList list;
				  QgsFeature f;
				  newLayer->select( spr->attributeIndexes(), QgsRectangle() );

				  while ( newLayer->nextFeature( f ) )
				  {
					  list.append( f );
				  }
				  if ( !pr->addFeatures( list ) )
				  {
					  qDebug() << "features not added";
				  }
				  qDebug() << "setting renderer";
					vl->setRenderer( newLayer->renderer()->clone() );

					qDebug() << "vl renderer: " << vl->renderer()->name() << vl->renderer();

				  qDebug() << "renderer setted";
				  QgsMapLayer* ml = QgsMapLayerRegistry::instance()->addMapLayer( vl );
				  myLayerSet.append(QgsMapCanvasLayer(ml, TRUE));
			  } else
			  {
				  qDebug( "Unable to load layer." );

			  }
			  qDebug() << "deleting newLayer";
			  delete newLayer;

		  }

	  }
  }
	qDebug() << "deleting doc";
  delete doc;

  qDebug() << "Set the Map Canvas Layer Set";
  mMapCanvas->setLayerSet(myLayerSet);
  qDebug() << "mMapCanvas->setVisible(true)";
  mMapCanvas->setVisible(true);
  qDebug() << "mMapCanvas->refresh()";
  mMapCanvas->refresh();

  // Start the Application Event Loop
  return app.exec();
}


